this.deadly_poison_effect <- this.inherit("scripts/skills/skill", {
	m = {
		Damage = 5,
		DamageBonusPerTurn = 3,
		LastRoundApplied = 0
	},
	function getDamage()
	{
		return this.m.Damage;
	}

	function setDamage( _d )
	{
		this.m.Damage = _d;
	}

	function create()
	{
		this.m.ID = "effects.deadly_poison";
		this.m.Name = "Poisoned";
		this.m.KilledString = "Died from poison";
		this.m.Icon = "skills/status_effect_deadly_poison.png";
		this.m.IconMini = "status_effect_54_mini";
		this.m.SoundOnUse = [
			"sounds/enemies/giant_spider_poison_01.wav",
			"sounds/enemies/giant_spider_poison_02.wav"
		];
		this.m.Type = this.Const.SkillType.StatusEffect | this.Const.SkillType.DamageOverTime;
		this.m.IsActive = false;
		this.m.IsStacking = true;
		this.m.IsRemovedAfterBattle = true;
	}

	function getDescription()
	{
		return "This character has a vicious poison running through his veins and will lose [color=" + this.Const.UI.Color.NegativeValue + "]" + this.m.Damage + "[/color] hitpoints each turn. Damage increases each turn by [color=" + this.Const.UI.Color.NegativeValue + "]" + this.m.DamageBonusPerTurn + "[/color]. Has no turns limit.";
	}

	function applyDamage()
	{
		if (this.m.LastRoundApplied != this.Time.getRound())
		{
			this.m.LastRoundApplied = this.Time.getRound();
			this.spawnIcon("status_effect_54", this.getContainer().getActor().getTile());

			if (this.m.SoundOnUse.len() != 0)
			{
				this.Sound.play(this.m.SoundOnUse[this.Math.rand(0, this.m.SoundOnUse.len() - 1)], this.Const.Sound.Volume.RacialEffect * 1.0, this.getContainer().getActor().getPos());
			}

			local hitInfo = clone this.Const.Tactical.HitInfo;
			hitInfo.DamageRegular = this.m.Damage;
			hitInfo.DamageDirect = 1.0;
			hitInfo.BodyPart = this.Const.BodyPart.Body;
			hitInfo.BodyDamageMult = 1.0;
			hitInfo.FatalityChanceMult = 0.0;
			this.getContainer().getActor().onDamageReceived(this.getContainer().getActor(), this, hitInfo);

			this.m.Damage += this.m.DamageBonusPerTurn;
		}
	}

	function onAdded()
	{
		if (this.getContainer().getActor().getCurrentProperties().NegativeStatusEffectDuration < 0)
		{
			this.m.DamageBonusPerTurn = 1;
		}

		if (this.getContainer().hasSkill("trait.ailing"))
		{
			this.m.DamageBonusPerTurn = 5;
		}
	}

	function onTurnEnd()
	{
		this.applyDamage();
	}

	function onWaitTurn()
	{
		this.applyDamage();
	}

});
